@@ -1,4 +1,5 @@
 from typing import List
+from tqdm import tqdm
 
 from .base import Engine
 from .task import Task
@@ -13,12 +14,18 @@ class SingleCoreEngine(Engine):
     def __init__(self):
         super().__init__()
 
-    def execute(self, tasks: List[Task]):
-        """
-        Execute all tasks in a simple for loop sequentially.
+    def execute(self, tasks: List[Task], progress_bar: bool = True):
+        """Execute all tasks in a simple for loop sequentially.
+
+        Parameters
+        ----------
+        tasks:
+            List of tasks to execute.
+        progress_bar:
+            Whether to display a progress bar.
         """
         results = []
-        for task in tasks:
+        for task in tqdm(tasks, disable=not progress_bar):
             results.append(task.execute())
 
         return results
