@@ -15,6 +15,8 @@ from .optimizer import (
     DlibOptimizer,
     PyswarmOptimizer,
     CmaesOptimizer,
+    PyswarmsOptimizer,
+    ScipyDifferentialEvolutionOptimizer,
     NLoptOptimizer,
     FidesOptimizer,
 )
