@@ -5,6 +5,19 @@ from .util import rescale
 def latin_hypercube(**kwargs) -> np.ndarray:
     """
     Generate latin hypercube points.
+
+    Parameters
+    ----------
+    n_starts:
+        number of starting points to be sampled.
+    lb:
+        lower bound.
+    ub:
+        upper bound.
+    smooth:
+        indicates if a (uniformly chosen) random starting point within the
+        hypercube [i/n_starts, (i+1)/n_starts] should be chosen (True) or
+        the midpoint of the interval (False). Default is True.
     """
     # extract input
     n_starts = kwargs['n_starts']
@@ -31,18 +44,31 @@ def latin_hypercube(**kwargs) -> np.ndarray:
 
 
 def _latin_hypercube(
-        n_starts: int, dim: int, smooth: bool = True
+        n_starts: int,
+        dim: int,
+        smooth: bool = True
 ) -> np.ndarray:
     """
     Generate simple latin hypercube points in [0, 1].
-    """
-    # uniform points
-    xs = np.random.random((n_starts, dim))
 
-    # assign sorted indices
-    for j_dim in range(0, dim):
-        indices = np.argsort(xs[:, j_dim])
-        xs[:, j_dim] = indices
+    Parameters
+    ----------
+
+    n_starts:
+        number of starting points to be sampled.
+
+    dim:
+        dimension of the optimization problem.
+
+    smooth:
+        indicates, if a (uniformly chosen) random starting point within the
+        hypercube [i/n_starts, (i+1)/n_starts] should be chosen
+        (`smooth==True`) or the midpoint of the interval (`smoth==False`).
+    """
+    xs = np.empty((n_starts, dim))
+
+    for i_dim in range(dim):
+        xs[:, i_dim] = np.random.permutation(np.arange(n_starts))
 
     if smooth:
         xs += np.random.random((n_starts, dim))
