@@ -4,7 +4,7 @@ from ..optimize.result import OptimizerResult
 from ..profile.result import ProfilerResult
 from ..sample.result import McmcPtResult
 from ..problem import Problem
-from ..objective import Objective, ObjectiveBase
+from ..objective import Objective, ObjectiveBase, Hdf5History
 import numpy as np
 import logging
 
@@ -44,6 +44,7 @@ def read_hdf5_profile(f: h5py.File,
 
 
 def read_hdf5_optimization(f: h5py.File,
+                           file_name: str,
                            opt_id: str) -> 'OptimizerResult':
     """
     Read HDF5 results per start.
@@ -52,6 +53,8 @@ def read_hdf5_optimization(f: h5py.File,
     -------------
     f:
         The HDF5 result file
+    file_name:
+        The name of the HDF5 file, needed to create HDF5History
     opt_id:
         Specifies the start that is read from the HDF5 file
     """
@@ -59,6 +62,12 @@ def read_hdf5_optimization(f: h5py.File,
     result = OptimizerResult()
 
     for optimization_key in result.keys():
+        if optimization_key == 'history':
+            if optimization_key in f:
+                result['history'] = Hdf5History(id=opt_id,
+                                                file=file_name)
+                result['history']._recover_options(file_name)
+                continue
         if optimization_key in f[f'/optimization/results/{opt_id}']:
             result[optimization_key] = \
                 f[f'/optimization/results/{opt_id}/{optimization_key}'][:]
@@ -154,7 +163,9 @@ class OptimizationResultHDF5Reader:
                 self.results.problem = problem_reader.read()
 
             for opt_id in f['/optimization/results']:
-                result = read_hdf5_optimization(f, opt_id)
+                result = read_hdf5_optimization(f,
+                                                self.storage_filename,
+                                                opt_id)
                 self.results.optimize_result.append(result)
             self.results.optimize_result.sort()
         return self.results
@@ -196,7 +207,11 @@ class SamplingResultHDF5Reader:
             for key in f['/sampling/results'].attrs:
                 sample_result[key] = \
                     f['/sampling/results'].attrs[key]
-        self.results.sample_result = McmcPtResult(**sample_result)
+        try:
+            self.results.sample_result = McmcPtResult(**sample_result)
+        except TypeError:
+            logger.warning("Warning: You tried loading a non-existent "
+                           "sampling result.")
 
         return self.results
 
@@ -232,16 +247,87 @@ class ProfileResultHDF5Reader:
                 problem_reader = ProblemHDF5Reader(self.storage_filename)
                 self.results.problem = problem_reader.read()
             for profile_id in f['/profiling']:
-                profiling_list.append([])
+                profiling_list.append([
+                    None for _ in f[f'/profiling/{profile_id}']
+                ])
                 for parameter_id in f[f'/profiling/{profile_id}']:
                     if f[f'/profiling/{profile_id}/'
                          f'{parameter_id}'].attrs['IsNone']:
-                        profiling_list[int(profile_id)].append(None)
-                    else:
-                        profiling_list[int(profile_id)]\
-                            .append(
-                            read_hdf5_profile(f,
-                                              profile_id=profile_id,
-                                              parameter_id=parameter_id))
+                        continue
+                    profiling_list[int(profile_id)][int(parameter_id)] = \
+                        read_hdf5_profile(f,
+                                          profile_id=profile_id,
+                                          parameter_id=parameter_id)
             self.results.profile_result.list = profiling_list
         return self.results
+
+
+def read_result(filename: str,
+                problem: bool = True,
+                optimize: bool = False,
+                profile: bool = False,
+                sample: bool = False,
+                ) -> Result:
+    """
+    This is a function that saves the whole pypesto.Result object in an
+    HDF5 file. With booleans one can choose more detailed what to save.
+
+    Parameters
+    ----------
+    filename:
+        The HDF5 filename.
+    problem:
+        Read the problem.
+    optimize:
+        Read the optimize result.
+    profile:
+        Read the profile result.
+    sample:
+        Read the sample result.
+
+    Returns
+    -------
+    result:
+        Result object containing the results stored in HDF5 file.
+    """
+    if not any([optimize, profile, sample]):
+        optimize = True
+        profile = True
+        sample = True
+    result = Result()
+
+    if problem:
+        pypesto_problem_reader = ProblemHDF5Reader(filename)
+        result.problem = pypesto_problem_reader.read()
+
+    if optimize:
+        pypesto_opt_reader = OptimizationResultHDF5Reader(filename)
+        try:
+            temp_result = pypesto_opt_reader.read()
+            result.optimize_result = temp_result.optimize_result
+        except KeyError:
+            logger.warning('Loading the optimization result failed. It is '
+                           'highly likely that no optimization result exists '
+                           f'within {filename}.')
+
+    if profile:
+        pypesto_profile_reader = ProfileResultHDF5Reader(filename)
+        try:
+            temp_result = pypesto_profile_reader.read()
+            result.profile_result = temp_result.profile_result
+        except KeyError:
+            logger.warning('Loading the profiling result failed. It is '
+                           'highly likely that no profiling result exists '
+                           f'within {filename}.')
+
+    if sample:
+        pypesto_sample_reader = SamplingResultHDF5Reader(filename)
+        try:
+            temp_result = pypesto_sample_reader.read()
+            result.sample_result = temp_result.sample_result
+        except KeyError:
+            logger.warning('Loading the sampling result failed. It is '
+                           'highly likely that no sampling result exists '
+                           f'within {filename}.')
+
+    return result
